# Lightweight OpenPose

## モデル

* [Lightweight OpenPose](https://github.com/Daniil-Osokin/lightweight-human-pose-estimation.pytorch)  
  * human-pose-estimation.onnx  

## 著作権/ライセンス

* Copyright (c) 2019 Daniil Osokin  
* [Apache License Version 2.0](https://github.com/Daniil-Osokin/lightweight-human-pose-estimation.pytorch/blob/master/LICENSE)  

## 変換方法

1. [PyTorch](https://pytorch.org/)のサイトで開発環境に合わせたコマンドを取得、インストールする。  
  （画像はWindowsでpipコマンドを利用してCPUで動作するPyTorchをインストールするコマンドを取得する例）  

  ![開発環境に合わせたPyTorchのインストールコマンドの例](pytorch.png)  

2. その他の変換に必要なパッケージ（ONNX、JSON、Protobuf）をインストールする。  

  ```bash
  pip install onnx json protobuf
  ```

3. [Lightweight OpenPose](https://github.com/Daniil-Osokin/lightweight-human-pose-estimation.pytorch)をクローンする。  

  ```bash
  git clone https://github.com/Daniil-Osokin/lightweight-human-pose-estimation.pytorch
  ```

4. [Lightweight OpenPose](https://github.com/Daniil-Osokin/lightweight-human-pose-estimation.pytorch)の学習済みモデルをダウンロードする。  

  * [checkpoint_iter_370000.pth](https://download.01.org/opencv/openvino_training_extensions/models/human_pose_estimation/checkpoint_iter_370000.pth)

5. 同梱されている**export.py**を実行してモデルを変換する。  
  ここではPyTorchの学習済みモデルをONNXフォーマットに変換しています。  

  ```python
  import os
  import torch
  from efficientnet_pytorch import EfficientNet

  def main():
      model_name = "efficientnet-b3" # b0-b7
      image_size = EfficientNet.get_image_size(model_name)
      print("input image size: {0}x{1}".format(image_size, image_size))

      pretrained_model = EfficientNet.from_pretrained(model_name)
      pretrained_model.set_swish(memory_efficient=False)
      pretrained_model.eval()

      dummy_input = torch.randn(10, 3, image_size, image_size)
      directory = os.path.dirname(__file__)
      onnx_file = os.path.join(directory, "{0}.onnx".format(model_name))
      torch.onnx.export(pretrained_model, dummy_input, onnx_file, verbose=True)
      print("[success] export onnx format {0}".format(onnx_file))

  if __name__ == '__main__':
      main()
  ```

6. 同梱されている**remove_layers.py**を実行してモデルを調整する。[^1]  
  ここではHigh Level API（`cv.cv.dnn_KeypointsModel`）で扱えるように余分なレイヤーを削除して出力レイヤーが1つになるように修正しています。  

  ```python
  import os
  import onnx
  import json
  from google.protobuf.json_format import MessageToJson
  from google.protobuf.json_format import Parse

  def main():
      directory = os.path.dirname(__file__)
      output_name = os.path.join(directory, "human-pose-estimation.onnx")
      onnx_model = onnx.load(output_name)
      json_str = MessageToJson(onnx_model)
      json_obj = json.loads(json_str)

      outputs = json_obj["graph"]["output"]
      remove_layers = ["stage_0_output_1_heatmaps", "stage_0_output_0_pafs", "stage_1_output_0_pafs"]
      json_obj["graph"]["output"] = [output for output in outputs if output["name"] not in remove_layers]

      nodes = json_obj["graph"]["node"]
      remove_nodes = ["Conv_114", "Relu_115", "Conv_116"]
      json_obj["graph"]["node"] = [node for node in nodes if node["name"] not in remove_nodes]

      json_str = json.dumps(json_obj)
      onnx_model = Parse(json_str, onnx.ModelProto())
      onnx.save(onnx_model, output_name)
      print("[success] remove unnecessary layers {0}".format(output_name))

  if __name__ == '__main__':
      main()
  ```

## Notebook

* [lightweight-openpose.ipynb | Google Colab](https://colab.research.google.com/drive/1GlRaMN8Oiu0tCB5HR_yy3lUXycffity1?usp=sharing)  

---
[^1]: Low Level APIでは出力レイヤーを指定できるため不要です。  
